from typing import Dict, List, Tuple

# Number that is used for inflation correction over taxes. Values gathered from the yearly document
# called "Belangrijkste wijzigingen belastingen XXXX" from Rijksoverheid
INFLATION = {
    2015: 0.009,
    2016: 0.005,
    2017: 0.003,
    2018: 0.008,
    2019: 0.012,
    2020: 0.016,
    2021: 0.016,
    2022: 0.013,
    2023: 0.063,
    2024: 0.099,
}

# Semi source https://financieel.infonu.nl/belasting/114283-wegenbelasting-2020-en-2021-oldtimer-omhoog.html
KWARTTARIEF_MAX = {
    2015: 120,
    2016: 120,
    2017: 120,
    2018: 123,
    2019: 125,
    2020: 125,
    2021: 129,
    2022: 130,
    2023: 138,
    2024: 152,
    2025: 154,
}

# Define weight tax brackets per energy source
WEIGHT_TAX_BRACKETS: Dict[str, List[Tuple[int, float]]] = {
    "benzine": [  # Used for most sources
        (500, 18.75),
        (600, 25.44),
        (700, 32.33),
        (800, 42.20),
        (900, 56.13),
    ],
    "lpg_g3": [  # Used for most sources
        (500, 18.75),
        (600, 25.44),
        (700, 32.33),
        (800, 42.20),
        (900, 56.13),
    ],
    "elektriciteit": [  # Used for most sources
        (500, 18.75),
        (600, 25.44),
        (700, 32.33),
        (800, 42.20),
        (900, 56.13),
    ],
    "overige": [  # For EnergySource.OVERIGE
        (500, 86.25),
        (600, 103.39),
        (700, 120.54),
        (800, 136.65),
        (900, 150.36),
    ],
    "diesel": [  # For EnergySource.DIESEL
        (500, 73.75),
        (600, 87.02),
        (700, 100.51),
        (800, 114.25),
        (900, 133.69),
    ],
}

EXCESS_RATES: Dict[str, float] = {
    "benzine": 15.09,
    "lpg_g3": 15.09,
    "elektriciteit": 15.09,
    "overige": 15.92,
    "diesel": 14.48,
    "opcenten": 11.68,
}

OPCENTEN_BRACKETS: list[Tuple[int, float]] = [
    (500, 14.5),
    (600, 17.33),
    (700, 20.40),
    (800, 26.98),
    (900, 34.12),
    (901, 45.81),  # Hack to get the base_rate for > cutfoff values
]


"""Percentages of base tax added to total tax amount per province

Source: https://www.cbs.nl/nl-nl/cijfers/detail/80889ned
"""

OPCENTEN = {
    "groningen": {
        2015: 88.4,
        2016: 88.9,
        2017: 88.6,
        2018: 89.3,
        2019: 90.4,
        2020: 91.8,
        2021: 93.3,
        2022: 94.5,
        2023: 95.7,
        2024: 95.7,
    },
    "friesland": {
        2015: 94.1,
        2016: 68.8,
        2017: 69.4,
        2018: 70.0,
        2019: 71.1,
        2020: 87.0,
        2021: 87.0,
        2022: 87.0,
        2023: 87.0,
        2024: 89.6,
    },
    "drenthe": {
        2015: 90.3,
        2016: 92.0,
        2017: 92.0,
        2018: 92.0,
        2019: 92.0,
        2020: 92.0,
        2021: 92.0,
        2022: 92.0,
        2023: 92.0,
        2024: 92.0,
    },
    "overijssel": {
        2015: 79.9,
        2016: 79.9,
        2017: 79.9,
        2018: 79.9,
        2019: 79.9,
        2020: 79.9,
        2021: 79.9,
        2022: 79.9,
        2023: 79.9,
        2024: 82.2,
    },
    "flevoland": {
        2015: 76.6,
        2016: 77.4,
        2017: 78.2,
        2018: 79.0,
        2019: 79.8,
        2020: 80.6,
        2021: 81.4,
        2022: 82.2,
        2023: 82.2,
        2024: 83.0,
    },
    "gelderland": {
        2015: 89.3,
        2016: 88.7,
        2017: 89.2,
        2018: 89.2,
        2019: 89.2,
        2020: 87.2,
        2021: 89.5,
        2022: 90.6,
        2023: 93.0,
        2024: 97.9,
    },
    "utrecht": {
        2015: 72.6,
        2016: 72.6,
        2017: 72.6,
        2018: 72.6,
        2019: 72.6,
        2020: 72.6,
        2021: 74.9,
        2022: 77.5,
        2023: 79.4,
        2024: 81.9,
    },
    "noord-holland": {
        2015: 67.9,
        2016: 67.9,
        2017: 67.9,
        2018: 67.9,
        2019: 67.9,
        2020: 67.9,
        2021: 67.9,
        2022: 67.9,
        2023: 67.9,
        2024: 77.4,
    },
    "zuid-holland": {
        2015: 95.0,
        2016: 92.0,
        2017: 91.4,
        2018: 90.4,
        2019: 90.4,
        2020: 90.4,
        2021: 90.4,
        2022: 91.8,
        2023: 95.7,
        2024: 98.7,
    },
    "zeeland": {
        2015: 78.3,
        2016: 82.3,
        2017: 82.3,
        2018: 82.3,
        2019: 89.1,
        2020: 89.1,
        2021: 89.1,
        2022: 82.3,
        2023: 82.3,
        2024: 84.4,
    },
    "noord-brabant": {
        2015: 76.1,
        2016: 76.1,
        2017: 76.1,
        2018: 76.1,
        2019: 76.1,
        2020: 78.0,
        2021: 78.4,
        2022: 79.6,
        2023: 80.8,
        2024: 82.8,
    },
    "limburg": {
        2015: 77.9,
        2016: 77.9,
        2017: 77.9,
        2018: 77.9,
        2019: 77.9,
        2020: 77.9,
        2021: 77.9,
        2022: 79.1,
        2023: 80.6,
        2024: 83.1,
    },
}
